﻿using Windows.UI.Xaml;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Media.Animation;

namespace Animations
{
    public class AnimationHelper : FrameworkElement
    {
        private static readonly DependencyProperty DoubleProxyProperty = DependencyProperty.Register(
            "DoubleProxy", 
            typeof(double), 
            typeof(AnimationHelper), 
            new PropertyMetadata(0.0, DoubleProxyChanged));

        private static readonly DependencyProperty AttachedAnimationProperty = DependencyProperty.Register(
            "AttachedAnimation",
            typeof(Timeline),
            typeof(AnimationHelper),
            new PropertyMetadata(null));

        private DependencyObject owner;
        private DependencyProperty doubleProperty;

        private AnimationHelper(DependencyObject owner, DependencyProperty doubleProperty)
        {
            this.owner = owner;
            this.doubleProperty = doubleProperty;

            // prevent this instance from being released prior to finishing the animation
            this.owner.SetValue(AttachedAnimationProperty, this);

            this.SetBinding(DoubleProxyProperty, new Binding() { Path = new PropertyPath("Opacity"), Source = this });
        }
        
        internal static void SetDoubleAnimation(DoubleAnimation animation, FrameworkElement owner, DependencyProperty doubleProperty)
        {
            AnimationHelper animationHelper = new AnimationHelper(owner, doubleProperty);
            Storyboard.SetTarget(animation, animationHelper);
            Storyboard.SetTargetProperty(animation, "Opacity");
        }

        private static void DoubleProxyChanged(DependencyObject target, DependencyPropertyChangedEventArgs args)
        {
            AnimationHelper animationProxy = (AnimationHelper)target;
            animationProxy.owner.SetValue(animationProxy.doubleProperty, args.NewValue);
        }
    }
}
